local AddonName, Addon = ...
local L = Addon:GetLocale()

local Package = select(2, ...);

-- Simple helper function which handles enumerating bags and running the function.
local function withEachBagAndItem(func, startBag, endBag)

    local itemproperties = Addon:GetSystem("ItemProperties")
    for bag=startBag, endBag do
        for slot=1, Addon:GetContainerNumSlots(bag) do
            local item = itemproperties:GetItemPropertiesFromBagAndSlot(bag, slot);
            if (item) then
                if not func(item, bag, slot) then
                    return false;
                end
            end
        end
    end
    return true;
end

--[[===========================================================================
    | GetMatchesForRule:
    |   Evaluate the given parameters against all the items your bag, this used
    |   to show matches in the edit rule dialog. The return value is a table with
    |   all the links in.  This bypasses the rule manager since we know exactly
    |   what we're evaluating.
    =============================================================================]]
function Addon:GetMatchesForRule(engine, ruleId, ruleScript, parameters)

    local rulesEngine = engine or self:CreateRulesEngine();
    local results = {};

    -- Make sure we've got a category
    rulesEngine:RemoveCategory(1);
    rulesEngine:CreateCategory(1, "<test>");

    local result, message = rulesEngine:AddRule(1, { Id = ruleId, Name = ruleId, Script = ruleScript }, parameters);
    if (result) then
        withEachBagAndItem(
            function(item, bag, slot)
                local result = rulesEngine:Evaluate(item);
                if (result) then
                    -- TODO: Change this to use item GUID for results and populate making item from GUID
                    -- Can also set tooltip by GUID now.
                    -- Do deep copy so that location doesn't potentially get tainted.
                    local locationCopy = Addon.DeepTableCopy(ItemLocation:CreateFromBagAndSlot(bag, slot))
                    table.insert(results, locationCopy);
                end
                return true;
           end, 0, Addon:GetNumTotalEquippedBagSlots() );
    else

    end


    return results;
end

--[[===========================================================================
    | ValidateRuleAgainstBags:
    |   Use the players bags to validate the rule, this hopefully catches anything
    |   wrong with rule.  I am assuming that they want to match an item contained
    |   inside of their bags.
    ===========================================================================--]]
function Addon:ValidateRuleAgainstBags(engine, script, parameters)

    local rulesEngine = engine or self:CreateRulesEngine();
    local message = "";
    local valid = withEachBagAndItem(
        function(item)
            local r, m = engine:ValidateScript(item, script, parameters);
            if (not r) then message = m end;
            return r;
        end, 0, Addon:GetNumTotalEquippedBagSlots() );

    return valid, message;
end

--[[===========================================================================
    | GetRuleStatus:
    |   Returns the status of rules
    ===========================================================================--]]
function Addon:GetRuleStatus(ruleId)
    if (not self.ruleManager or not self.ruleManager.rulesEngine) then
        return nil;
    end

    local status = self.ruleManager.rulesEngine:GetRuleStatus(ruleId);
    if (status and (#status == 1)) then
        return unpack(status[1]);
    end
end

--[[===========================================================================
    | LookForItemsInBank:
    |   Evaluates all of the items in the bank against the rules, and returns
    |   the matches.
    ===========================================================================--]]
function Addon:LookForItemsInBank()
    if (not self.ruleManager) then
        self.ruleManager = Addon.RuleManager:Create();
    end

    local items = {};
    withEachBagAndItem(
        function (item, bag, slot)
            local sell = self.ruleManager:Run(item);
            if (sell) then
                table.insert(items, { bag, slot, item.Link });
            end
            return true;
        end,
        (Addon:GetNumTotalEquippedBagSlots()  + 1),  (Addon:GetNumTotalEquippedBagSlots()  + GetNumBankSlots()));
    return items;
end

--[[===========================================================================
    | GetRulesEngine:
    |   Retrieves the initialized rules engine for this instance.
    =======================================================================--]]
function Addon:GetRuleManager()
    if (not self.ruleManager) then
        self.ruleManager = Addon.RuleManager:Create();
    end
    return self.ruleManager;
end
